@@ -1,7 +1,7 @@
 package Tree::RB::XS;
-$Tree::RB::XS::VERSION = '0.02';
+$Tree::RB::XS::VERSION = '0.03';
 # VERSION
-# ABSTRACT: Similar API to Tree::RB implemented in C
+# ABSTRACT: Red/Black Tree implemented in C, with similar API to Tree::RB
 
 use strict;
 use warnings;
@@ -18,6 +18,7 @@ our %EXPORT_TAGS= (
 	key_type => \@_key_types,
 	cmp      => \@_cmp_enum,
 	lookup   => \@_lookup_modes,
+	get      => \@_lookup_modes,
 	all      => \@EXPORT_OK,
 );
 
@@ -40,19 +41,19 @@ sub new {
 
 
 sub iter {
-	my $self= shift;
-	my $node= @_? $self->get_node($_[0], GET_GE()) : $self->min;
-	bless sub { my $x= $node; $node= $node->next if $node; $x }, 'Tree::RB::XS::Iter';
+	my ($self, $key_or_node, $mode)= @_;
+	$key_or_node= $self->get_node($key_or_node, @_ > 2? $mode : GET_GE())
+		if @_ > 1 && ref $key_or_node ne 'Tree::RB::XS::Node';
+	Tree::RB::XS::Iter->_new($key_or_node || $self, 1);
 }
 
 sub rev_iter {
-	my $self= shift;
-	my $node= @_? $self->get_node($_[0], GET_LE_LAST()) : $self->max;
-	bless sub { my $x= $node; $node= $node->prev if $node; $x }, 'Tree::RB::XS::Iter';
+	my ($self, $key_or_node, $mode)= @_;
+	$key_or_node= $self->get_node($key_or_node, @_ > 2? $mode : GET_LE_LAST())
+		if @_ > 1 && ref $key_or_node ne 'Tree::RB::XS::Node';
+	Tree::RB::XS::Iter->_new($key_or_node || $self, -1);
 }
 
-sub Tree::RB::XS::Iter::next { shift->() }
-
 
 *Tree::RB::XS::Node::min=         *Tree::RB::XS::Node::left_leaf;
 *Tree::RB::XS::Node::max=         *Tree::RB::XS::Node::right_leaf;
@@ -82,6 +83,33 @@ sub Tree::RB::XS::Node::as_lol {
 	]
 }
 
+sub Tree::RB::XS::Node::iter {
+	Tree::RB::XS::Iter->_new($_[0], 1);
+}
+
+sub Tree::RB::XS::Node::rev_iter {
+	Tree::RB::XS::Iter->_new($_[0], -1);
+}
+
+
+# I can't figure out how to do the closure in XS yet
+sub Tree::RB::XS::Iter::_new {
+	my $class= shift;
+	my ($self,$y);
+	$self= bless sub { Tree::XS::RB::Iter::next($y) }, $class;
+	Scalar::Util::weaken($y= $self);
+	$self->_init(@_);
+}
+sub Tree::RB::XS::Iter::clone {
+	my $self= shift;
+	ref($self)->_new($self);
+}
+
+
+*TIEHASH= *new;
+*STORE= *put;
+*CLEAR= *clear;
+
 
 *LUEQUAL= *GET_EQ;
 *LUGTEQ=  *GET_GE;
@@ -91,6 +119,7 @@ sub Tree::RB::XS::Node::as_lol {
 *LUPREV=  *GET_PREV;
 *LUNEXT=  *GET_NEXT;
 
+
 1;
 
 __END__
@@ -101,13 +130,15 @@ __END__
 
 =head1 NAME
 
-Tree::RB::XS - Similar API to Tree::RB implemented in C
+Tree::RB::XS - Red/Black Tree implemented in C, with similar API to Tree::RB
 
 =head1 SYNOPSIS
 
 B<NOTICE:> This module is very new and you should give it some thorough testing before
 trusting it with production data.
 
+  use Tree::RB::XS qw/ :key_type :get /;
+  
   my $tree= Tree::RB::XS->new;
   $tree->put($_ => $_) for 'a'..'z';
   say $tree->get('a');
@@ -122,7 +153,7 @@ trusting it with production data.
 
   # optimize for floating-point comparisons
   $tree= Tree::RB::XS->new(key_type => 'float', allow_duplicates => 1);
-  $tree->put(rand() => 1);
+  $tree->put(rand() => 1) for 1..1000;
   $tree->delete(0, $tree->get_node_lt(.5));
 
   # optimize for byte strings
@@ -176,7 +207,7 @@ Many functions have official names changed, but aliases are provided for compati
 =head2 new
 
   my $tree= Tree::RB::XS->new( %OPTIONS );
-                     ...->new( sub($x,$y) { $x cmp $y });
+                     ...->new( \&compare_fn );
 
 Options:
 
@@ -186,13 +217,16 @@ Options:
 
 Choose an optimized storage for keys.  The default is L</KEY_TYPE_ANY> which stores
 whole perl scalars.  All the other types are faster, but perl scalars give the fewest
-surprises.
+surprises.  If importing constants is annoying, you can specify one of C<"any">,
+C<"int">, C<"float">, C<"bstr">, or C<"ustr">.
 
 =item L</compare_fn>
 
-Choose a custom key-compare function.  The default depends on C<key_type>.
-If this is a perl coderef, the C<key_type> is forced to be L</KEY_TYPE_ANY>.
-Avoid using a coderef if possible.
+Choose a custom key-compare function.  The default is C<CMP_PERL> which gives
+you the behavior of Perl's C<cmp> operator.
+You can supply a custom coderef, but avoid doing that if you are concerned about
+speed; consider pre-processing your keys instead, to be able to use one of the
+built-in compare functions.
 
 =item L</allow_duplicates>
 
@@ -214,15 +248,14 @@ the constructor.
 
 This is one of the following values: L</KEY_TYPE_ANY>, L</KEY_TYPE_INT>,
 L</KEY_TYPE_FLOAT>, L</KEY_TYPE_BSTR>, or L</KEY_TYPE_USTR>.
-See the description in EXPORTS for full details on each.  If importing constants
-is annoying, you can specify these simply as C<"any">, C<"int">, C<"float">,
-C<"bstr">, and C<"ustr">.
+See the description in EXPORTS for full details on each.
 
 Integers are of course the most efficient, followed by floats, followed by
 byte-strings and unicode-strings, followed by 'ANY' (which stores a whole
-perl scalar).  BSTR and USTR both save an internal copy of your key, so
-might be a bad idea if your keys are extremely large and nodes are frequently
-added to the tree.
+perl scalar).  BSTR and USTR are both faster than operating on Perl scalars,
+but both save an internal copy of your key and might be a bad idea if your
+keys are extremely large and nodes are frequently added to the tree.
+(modern Perls can copy-on-write to avoid copying the key)
 
 =head2 compare_fn
 
@@ -238,8 +271,9 @@ Note that this forces C<< key_type => KEY_TYPE_ANY >>.
 Beware that using a custom coderef throws away most of the speed gains from using
 this XS variant over plain L<Tree::RB>.
 
-If not provided, the default comparison is chosen to match the C<key_type>,
-with the defult C<KEY_TYPE_ANY> using Perl's own C<cmp> comparator.
+The default comparison is C<CMP_PERL> which is equivalent to Perl's own C<cmp>
+comparator.  If you specify a C<key_type> and not a C<compare_fn>, the default
+comparator depends on the C<key_type>.
 
 Patches welcome, for anyone who wants to expand the list of optimized built-in
 comparison functions.
@@ -293,20 +327,21 @@ Alias: C<nth>
 =head2 get
 
   my $val= $tree->get($key);
-                ->get($key, $mode);
+             ...->get($key, $mode);
 
-Fetch a value from the tree, by its key.  Unlike L<Tree::RB/get>, this always
-returns a single value, regardless of list context.  But, you can set
-L<compat_list_get> to make C<get> an alias for C<lookup>.
+Fetch a value from the tree, by its key.  Unlike L<Tree::RB/get>, this returns a single
+value, regardless of list context.  But, you can set L<compat_list_get> to make C<get>
+an alias for C<lookup>.
 
 Mode can be used to indicate something other than an exact match:
 L</GET_EQ>, L</GET_EQ_LAST>, L</GET_LE>, L</GET_LE_LAST>, L</GET_LT>, L</GET_GE>, L</GET_GT>.
+(described below)
 
 =head2 get_node
 
 Same as L</get>, but returns the node instead of the value.  In trees with
-duplicate keys, this always returns the first node.  (nodes with identical keys
-are preserved in the order they were added)
+duplicate keys, this returns the first matching node.
+(nodes with identical keys are preserved in the order they were added)
 
 Aliases with built-in mode constants:
 
@@ -347,8 +382,8 @@ You can also use Tree::RB's lookup-mode constants of "LUEQUAL", etc.
 
 Associate the key with a new value.  If the key previously existed, this returns
 the old value, and updates the tree to reference the new value.  If the tree
-allows duplicate keys, this will replace all nodes having this key (but only return
-one of them).
+allows duplicate keys, this will remove all but one node having this key and
+then set its value.  Only the first old value will be returned.
 
 =head2 insert
 
@@ -360,43 +395,47 @@ node after all nodes of the same key, preserving the insertion order.
 =head2 delete
 
   my $count= $tree->delete($key);
-                  ->delete($key1, $key2);
-                  ->delete($node1, $node2);
-                  ->delete($start, $tree->get($limit, GET_LT));
+               ...->delete($key1, $key2);
+               ...->delete($node1, $node2);
+               ...->delete($start, $tree->get_node_lt($limit));
 
 Delete any node with a key identical to C<$key>, and return the number of nodes
 removed.  If you supply two keys (or two nodes) this will delete those nodes and
 all nodes inbetween; C<$key1> is searched with mode C<GET_GE> and C<$key2> is
 searched with mode C<GET_LE>, so the keys themselves do not need to be found in
 the tree.
-The keys (or nodes) most be given in ascending order, else no nodes are deleted.
+The keys (or nodes) must be given in ascending order, else no nodes are deleted.
 
 If you want to delete a range *exclusive* of one or both ends of the range, just
-use the C</get> method with the desired mode to look up each end of the nodes that
-you do want removed.
+use the C</get_node> method with the desired mode to look up each end of the nodes
+that you do want removed.
+
+=head2 clear
+
+  my $count= $tree->clear();
+
+This is the fastest way to remove all nodes from the tree.  It gets to destroy all
+the nodes without worrying about the tree structure or shifting iterators aside.
 
 =head2 iter
 
-  my $iter= $tree->iter;
-                 ->iter($from_key);
-  while (my $node= $iter->()) { ... }
-  while (my $node= $iter->next) { ... }
+  my $iter= $tree->iter;                              # from min_node
+              ...->iter($from_key, $get_mode=GET_GE); # from get_node
+              ...->iter($from_node);                  # from existing node
 
-Return an iterator object that traverses the tree.  The iterator is a blesed coderef, so you
-can either call it as a fuction or call the C<< ->next >> method.  If the C<$from_key> is
-provided, this starts from C<< $tree->get($key, GET_GE) >>.
+Return an L<iterator object|ITERATOR OBJECTS> that traverses the tree from min to max,
+or from the key or node you provide up to max.
 
 =head2 rev_iter
 
-Like C<iter>, but the C<< ->next >> method walks backward to smaller key values, and if the
-initial value is provided and duplicate keys are enabled, this starts on the right-most match
-of the key instead of the left-most match.
+Like C<iter>, but the C<< ->next >> and C<< ->step >> methods walk backward to smaller key
+values, and the default C<$get_mode> is L</GET_GE_LAST>.
 
 =head1 NODE OBJECTS
 
-The nodes returned by the methods above have the following attributes:
+=head2 Node Attributes
 
-=over 10
+=over
 
 =item key
 
@@ -450,9 +489,9 @@ The number of items in the tree rooted at this node (inclusive)
 
 =back
 
-And the following methods:
+=head2 Node Methods
 
-=over 10
+=over
 
 =item prune
 
@@ -469,8 +508,144 @@ For compat with L<Tree::RB::Node/strip>.
 Return sub-tree as list-of-lists. (array of arrays rather?)
 For compat with L<Tree::RB::Node/as_lol>.
 
+=item iter
+
+Shortcut for C<< $node->tree->iter($node) >>.
+
+=item rev_iter
+
+Shortcut for C<< $node->tree->rev_iter($node) >>.
+
 =back
 
+=head1 ITERATOR OBJECTS
+
+Iterators are similar to Nodes, but they hold a strong reference to the tree, and if a node
+they point to is removed from the tree they advance to the next node.  (and obviously they
+iterate, where node objects do not)
+
+The iterator references a "current node" which you can inspect the key and value of.  You
+can call 'step' to move to a new current node, and you can call 'next' which returns the
+current node while switching the reference to the next node.
+
+Note that if you avoid referencing the Node, and stick to the attributes and methods of the
+iterator, the tree can avoid allocating the Perl object to represent the Node.  This gives a
+bit of a performance boost for large tree operations.
+
+=head2 Iterator Attributes
+
+=over
+
+=item key
+
+The key of the current node.
+
+=item value
+
+The value of the current node.  Note that this returns the actual value, which in an aliased
+context allows you to modify the value stored in the tree.
+
+  $_++ for $iter->value;
+
+=item index
+
+The index of the current node.
+
+=item tree
+
+A reference back to the Tree.  Note that each iterator holds a strong reference to the tree.
+
+=item done
+
+True if the iterator has reached the end of its sequence, and no longer references a
+current Node.
+
+=back
+
+=head2 Iterator Methods
+
+=over
+
+=item next
+
+  my $nodes= $iter->next;
+  my @nodes= $iter->next($count);
+  my @nodes= $iter->next('*');
+
+Return the current node (as a L<node object|/NODE OBJECTS>) and advance to the following node
+in the sequence.  After the end of the sequence, calls to C<next> return C<undef>.
+If you pass the optional C<$count>, it will return up to that many nodes, as a list.
+It will also return an empty list at the end of the sequence instead of returning C<undef>.
+You can use the string C<'*'> for the count to indicate all the rest of the nodes in the
+sequence.
+
+=item next_keys
+
+Same as C<next>, but return the keys of the nodes.
+
+=item next_values
+
+Same as C<next>, but return the values of the nodes.  Like L</value>, these are also aliases,
+and can be modified.
+
+  $_++ for $iter->next_values('*');
+
+=item next_kv
+
+  my %x= $iter->next_kv('*');
+
+Same as C<next>, but return pairs of key and value for each node.  This is useful for dumping
+them into a hash. (B<unless> you have duplicate keys enabled, then don't dump them into a hash
+or you would lose elements)
+
+=item step
+
+    $iter->step;     # advance by one node
+	$iter->step(10); # advance by 10 nodes
+	$iter->step(-4); # back up 4 nodes
+
+This moves the iterator by one or more nodes in the forward or backward direction.
+For a reverse-iterator, positive numbers move toward the minimum key and negative numbers
+move toward the maximum key.  If the offset would take the iterator beyond the last node,
+the current node becomes C<undef>.  If the offset would take the iterator beyond the first
+node, the first node becomes the current node.
+
+=item delete
+
+Delete the "current node", return its value, and advance to the next node.
+
+  for (my $i= $tree->iter; !$i->done;) {
+    if ($i->key =~ ...) {
+      say "Removing ".$i->key." = ".$i->delete;
+    } else {
+      $i->step;
+    }
+  }
+
+This is useful when combined with the C<key> and C<value> attributes of the iterator, but not
+so much when you are looping using C<next>, because C<next> has already moved to the next node
+beyond the one it returned to you.  When using C<next>, call C<delete> on the node, not the
+iterator.
+
+=item clone
+
+Returns a new iterator of the same direction pointing at the same node.
+
+=back
+
+=head1 TIE HASH INTERFACE
+
+This class implements the required methods needed for C<tie>:
+
+  my %hash
+  my $tree= tie %hash, 'Tree::RB::XS';
+  $hash{$_}= $_ for 1..10;
+  delete $hash{3};
+  $_ += 1 for values %hash;
+
+But you get better performance by using the tree's API directly.  This should only be used when
+you need to integrate with code that isn't aware of the tree.
+
 =head1 EXPORTS
 
 =head2 Key Types
@@ -522,11 +697,13 @@ Perl Scalars.
 
 =item CMP_INT
 
-Compare keys directly as C integers.  This is the fastest option.
+Compare keys directly as whatever integer type Perl was compiled with.
+(i.e. 32-bit or 64-bit)  This is the fastest option.
 
 =item CMP_FLOAT
 
-Compare the keys directly as C 'double' values.
+Compare the keys directly as whatever floating-point type Perl was compiled with.
+(i.e. 64-bit double or 80-bit long double)
 
 =item CMP_UTF8
 
@@ -543,13 +720,11 @@ Compare using the equivalent of this coderef:
   sub {
     my @a_parts= split /([0-9]+)/, $_[0];
     my @b_parts= split /([0-9]+)/, $_[1];
-    my $i= 0;
-    while ($i < @a_parts || $i < @b_parts) {
+    for (my $i= 0; $i < @a_parts || $i < @b_parts; $i++) {
       no warnings 'uninitialized';
       my $cmp= ($i & 1)? ($a_parts[$i] <=> $b_parts[$i])
              : ($a_parts[$i] cmp $b_parts[$i]);
       return $cmp if $cmp;
-      ++$i;
     }
     return 0;
   }
@@ -637,13 +812,25 @@ Has alias C<LUPREV> to match Tree::RB.
 
 =back
 
+=head1 BUGS AND LIMITATIONS
+
+=over
+
+=item *
+
+Before Perl 5.14, there isn't any built-in utf8-aware case-sensitive compare function, so
+requests for C<CMP_UTF8> get changed to C<CMP_PERL> (with C<KEY_TYPE_ANY>) and C<CMP_NUMSPLIT>
+loses UTF8-awareness.
+
+=back
+
 =head1 AUTHOR
 
 Michael Conrad <mike@nrdvana.net>
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 COPYRIGHT AND LICENSE
 
